/*
Copyright (c) 2000-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: Implementation of the RCountSubSession class 
*/




#include "ComplexClientAndServer.h"
#include "ComplexClient.h"



/**
Creates a new subsession with the count server.
*/
TInt RCountSubSession::Open(RCountSession &aSession)
    {
    return CreateSubSession(aSession,ECountServCreateSubSession);
	}


/**
Close the subsession.

This just a simple wrapper around the 
RSubSessionBase class function that does the actual close operation.
*/
void RCountSubSession::Close()
	{
	RSubSessionBase::CloseSubSession(ECountServCloseSubSession);
	}


/**
A server request to initialise the subsession counter 
using a descriptor (e.g passing the text string "12").
*/
TInt RCountSubSession::SetFromString(const TDesC& aString)
	{
	TIpcArgs args(&aString);
    return SendReceive(ECountServInitSubSession, args);
	}


/**
A server request to increase the counter value by 
the default value (i.e. 1).
*/
void RCountSubSession::Increase()  
	{
	if (SubSessionHandle())
		{
		SendReceive(ECountServIncrease);
		}
	}


/**
A server request to increase the counter value by the specified
integer value.
*/
void RCountSubSession::IncreaseBy(TInt anInt)
	{
	if (SubSessionHandle())
		{
		TIpcArgs args(anInt);
	    SendReceive(ECountServIncreaseBy, args);
		}
	}


/**
A server request to decrease the counter value by
the default value (i.e. 1).
*/
void RCountSubSession::Decrease() 
	{
	if (SubSessionHandle())
		{
		SendReceive(ECountServDecrease);
		}
	}


/**
A server request to decrease the counter value by the specified
integer value.
*/
void RCountSubSession::DecreaseBy(TInt anInt)
	{
	if (SubSessionHandle())
		{
		TIpcArgs args(anInt);
	    SendReceive(ECountServDecreaseBy, args);
		}
	}


/**
A server request to reset the counter value to 0.
*/
void RCountSubSession::Reset()
	{
	if (SubSessionHandle())
		{
		SendReceive(ECountServReset);
		}
	}


/**
A server request to get the curent value of the counter.

We pass a TPckgBuf across to the server.
*/
TInt RCountSubSession::CounterValue()
	{
	TInt res = KErrNotFound;
	TPckgBuf<TInt> pckg;
	
	if (SubSessionHandle())
		{
		  // Note that TPckgBuf is of type TDes8
	    TIpcArgs args(&pckg);
	    SendReceive(ECountServValue, args);
		
		  // Extract the value returned from the server. 
	    res = pckg();
		}
		
    return res;        
	}



